#!/usr/bin/python
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################


#
# Import the tablon module
#
import tablon


# For hostname
import socket

#
# This class is passed to the tablon server
#
class CONFIG:

	# Just copy the hostname in the CONFIG
	hostname = socket.gethostname()

	# If the hostname is not one of the machines we are at the test host.
	if hostname not in ['machine1','machine2']:
		hostname = 'test'

	# Tablón version
	# It is important because the .js and .css files include the
	# name of the version in the url has a get parameter to
	# avoid caching when a new version is released
	version = '5.4'

	# Where to serve (by default)
	host='127.0.0.1'

	# Change the host based on the hostname (for infor.uva.es)
	if hostname != 'test':
		host =  hostname + '.your.domain.com'

	# Main color
	color = '#40C350' # Verde prueba

	# Color & favicon For a particular macine
	if hostname == 'machine1':
		color = '#1f8dd6' # Azul
		favicon = 'favicon_azul.ico'

	# Port for both sending files and web server
	port=8080
	
	# Port for the https server
	sport=4443

	# Show debug messages
	verbose=False

	# HTTP frontend number of threads
	# The frontend handles web and launch request.
	# If this is more than 1, it runs another implementation of the HTTP server
	# that uses a pool of threads to allow simultaneous user requests.
	frontend_threads = 4

	# Timeout for http connections
	http_timeout = 15
	if hostname == 'test':
		http_timeout = 60

	# Supported client version
	client_version = '1.6'

	# Send a compressed gzip auto executable python script
	# It has half the size and it is harder to hack.
	compress_client = True

	# Max size of the code
	max_program_size = 15000 * 10 # This is 10 times the size of my mpi version

	# The minimun time between user requests in seconds
	time_between_requests = 20
	if hostname == 'test':
		time_between_requests = 2

	# The minimun time between user requests in seconds
	# If they have used all the quota
	# (0 means no request are permitted)
	time_between_requests_no_quota = 60

	# Stop receiving requests X secons before the restart
	time_before_restart = 60

	# Number of programs in the main page
	requestnumber = 20

	# Size of the stdout and stderr
	maxstdoutsize = 2500 

	# Forbidden code in the programs
	forbidden=[
		'fopen',
		'setjmp',
		'FILE',
		'remove[(]',
		'rename[(]',
		'system[(]',
		'getenv[(]',
		'MPI_File_open',
		'sys/syscall.h',
		'sys/stat.h',
		'fstream'
	]

	# Admin users: they do not have time restrictions
	# and they can send a program to the leaderboard as reference
	# This user names may not exists
	admins = ['admin1','teacher1','teacher2']

	# Enable the admin pages
	# It uses https secure connections
	# but if not used it is better to disable it
	adminpages = False

	# Users and passwords of the admin pages
	adminpass = [('admin','useAbetterP@ssword')]
	if hostname == 'test':
		adminpass.append(('test','test'))

	# Add code to the user source
	addtosrc = '#include "cputils.h" // Added by tablon';

	# Expire html content, used for files such as css or js (in hours)
	expire_html = 10


#
# List of extra files to compile
# We add them to any queue so they can be copied
#
extrafiles = ['cputils.h','cputilstablon.h']
# Using a tuple, we can change the name
if CONFIG.hostname == 'machine1':
	extrafiles = ['cputils.h',('cputilstablon_m1.h','cputilstablon.h')]

# The previous line copies the cputils.h file from the makefile folder
# to the compile folder. It also copies the cputilstablon_m1.h and
# rename it to cputilstablon.h

if CONFIG.hostname == 'machine2':
	extrafiles = ['cutil32.lib','libcutil.a','libcutilD.a','cutil.h','cputils.h',('cputilstablon_m2.h','cputilstablon.h')]




#
# Create the queues
#

# Sequential
q_seq = tablon.LocalSeqQueue('sequential',
	extensions=['.c','.cpp'],makefile='Makefile_sequential',
	extrafiles=extrafiles,
	description='Sequential queue')





if CONFIG.hostname in ['test']:
	# Leaderboard suma
	lb_suma = q_seq.addLeaderboard('sum',
		description=u"The program receives several inputs as arguments and it should return the sum.")

	# Phases
	lb_suma.addPhase({'arguments':'1 1'},'2')       # 1 + 1 = 2
	lb_suma.addPhase({'arguments':'10 10 10'},'30') # 10 + 10 + 10 = 30
	lb_suma.addPhase({'arguments':'1 2 9'},'12')    # 1 + 2 + 9 = 12








if CONFIG.hostname in ['test','machine1']:

	# OpenMP queue
	q_omp = tablon.LocalOpenMPQueue('openmp', makefile='Makefile_openmp',
		threads=16,timewall=60*2,
		description="OpenMP queue")

	# Leaderboard that uses the previous queue
	lb_cadenas = q_omp.addLeaderboard('cadenasomp',isopen=False)

	# A leaderboard has several phases to test the program.
	# Each phase define the input and expected output.
	# The first ones do not have much load so we do not waste time for wrong programs.
	lb_cadenas.addPhase({'nprocs':1,'threads':14,'arguments':'Smallinput1 Smallinput2'},[0,1])
	lb_cadenas.addPhase({'nprocs':1,'threads':14,'arguments':'Biginput1  Biginput2 Worstcase'},[1123,123,-1])



	# MPI queue
	q_mpi = tablon.LocalMPIQueue('mpi', makefile='Makefile_mpi',
		nprocs=16,timewall=60*2,
		description="MPI programs queue")

	# You can create a leaderboard for MPI here in the same way as OpenMP

	# Hybrid queue
	q_hybrid = tablon.LocalMPIOpenMPQueue('hybrid',makefile='Makefile_hybrid',
		nprocs=16,threads=16,nprocs_x_threads=16,timewall=60*2,
		description='Queue for MPI+OpenMP')



# GPU test queue for the test machine
if CONFIG.hostname == 'test':

	# Cola CUDA
	tablon.LocalSeqQueue('cuda',extensions=['.cu'], makefile='Makefile_cuda',timewall=60*2)




# Machine2 has gpus and uses slurm
if CONFIG.hostname == 'machine2':


	# A queue that sends the programs to slurm
	tablon.SlurmQueue('cudaslurm',extensions=['.cu'], makefile='Makefile_cuda', nprocs=1, threads=1, slurm_usenodes=True)

	# Another queue that sends the programs to slurm but in a different slurm queue
	q_cuda = tablon.SlurmQueue('cudaboard',extensions=['.cu'], makefile='Makefile_cuda', nprocs=1, threads=1, slurm_queue='lboard')

	# MPI queue using slurm system
	q_mpi = tablon.SlurmQueue('mpi',extensions=['.c'], makefile='Makefile_mpi', nprocs=16, threads=1, slurm_queue='mpi')



	# A leaderboar for CUDA in the q_cuda queue
	lb_cudacadenas = q_cuda.addLeaderboard('cadenasCUDA',isopen=True)
	lb_cadenas.addPhase({'nprocs':1,'arguments':'Smallinput1 Smallinput2'},[0,1])
	lb_cadenas.addPhase({'nprocs':1,'arguments':'Biginput1  Biginput2 Worstcase'},[1123,123,-1])



#
# Create and run the server
#
tablon.run(CONFIG)

